
import plone.z3cform
import z3c.form
from Products.CMFCore.utils import getToolByName
from Products.statusmessages.interfaces import IStatusMessage
from collective.contentreports.tool import get_report
from collective.contentreports.tool import get_reports
from datetime import datetime
# from plone.formwidget.contenttree import ContentTreeFieldWidget
# from plone.formwidget.contenttree import PathSourceBinder
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile as FiveViewPageTemplateFile
from z3c.form.validator import WidgetValidatorDiscriminators
from zope import schema
from zope.component.hooks import getSite
from zope.app.pagetemplate import ViewPageTemplateFile
from zope.interface import Interface


def default_recipients():
    site = getSite()
    portal_membership = getToolByName(site, 'portal_membership')
    member = portal_membership.getAuthenticatedMember()
    email = member.getProperty('email')
    if email:
        return (email, )
    return ()


class IReportRequestSchema(Interface):

    report = schema.Choice(source='collective.contentreports.available_reports',
                           title=u'Select Report')

    recipients = schema.Tuple(value_type=schema.ASCIILine(title=u'Email'),
                              title=u'Email Report To',
                              description=(u'If you\'d like to send the report '
                                           u'to multiple people, type each '
                                           u'email on a separate line.'),
                              defaultFactory=default_recipients)

    # scope = schema.Choice(title=u'Report Scope',
    #                       source=PathSourceBinder(portal_type='Folder'))


class ReportRequestForm(z3c.form.form.Form):

    fields = z3c.form.field.Fields(IReportRequestSchema)

    ignoreContext = True

    label = u'Website Reports'

    # description = (
    #     u'It will take some time to generate the report you select. When '
    #     u'complete, the report will be emailed to the address(es) you specify. '
    #     u'PLEASE NOTE: that reports are queued up in the order that they are '
    #     u'received so you may not receive your report right away.')

    @z3c.form.button.buttonAndHandler(u'Run Report', name='run-report')
    def submit_request(self, action):
        context = self.context
        request = self.request
        data, errors = self.extractData()

        if len(errors):
            self.status = self.formErrorsMessage
            return

        site = getSite()
        tool = getToolByName(site, 'content_reports_tool')
        portal_membership = getToolByName(site, 'portal_membership')
        authenticated = portal_membership.getAuthenticatedMember()
        data['requester_email'] = authenticated.getProperty('email')
        data['requester_name'] = authenticated.getProperty('fullname')
        data['requester_id'] = authenticated.getId()
        context_state = context.restrictedTraverse('@@plone_context_state')
        folder = context_state.folder()
        data['scope'] = '/'.join(folder.getPhysicalPath())
        data['url'] = folder.absolute_url()
        data['timestamp'] = datetime.now()
        tool.schedule_report(**data)
        status = IStatusMessage(request)
        status.add(u'Report request submitted.', type=u'info')
        request.response.redirect(context.absolute_url())

    @z3c.form.button.buttonAndHandler(u'Cancel')
    def cancel(self, action):
        self.request.response.redirect(self.context.absolute_url())

    def updateWidgets(self):
        # self.fields['scope'].widgetFactory = ContentTreeFieldWidget

        super(ReportRequestForm, self).updateWidgets()

        # widget = self.widgets['report']
        # widget.onchange = u'displayReportDescription(this.value)'
        # widget.template = ViewPageTemplateFile('report_select_input.pt')

ReportRequestFormView = plone.z3cform.layout.wrap_form(
    ReportRequestForm,
    index=FiveViewPageTemplateFile('report_selection.pt'))


class ReportRequestUtils:

    def report_description(self):
        """ """
        request = self.request
        request.response.setHeader('Content-Type', 'text/Plain')
        reportid = request.get('selected_report')
        if reportid:
            report = get_report(reportid)
            if report is not None:
                return report.description
        else:
            reports = get_reports()
            if len(reports):
                return reports[0].description
        return ''
