
import traceback
from AccessControl import ClassSecurityInfo
from App.class_init import InitializeClass
from OFS.SimpleItem import SimpleItem
from Products.CMFCore.utils import UniqueObject
from Products.CMFCore.utils import getToolByName
from email.mime.text import MIMEText
from persistent.list import PersistentList
from persistent.mapping import PersistentMapping
from plone import api
from zope.component.hooks import getSite
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary

REPORTS = dict()


def register_report(report):
    REPORTS[report.name] = report


def get_report(name):
    return REPORTS.get(name, None)


def get_reports():
    keys = REPORTS.keys()
    keys.sort()
    reports = list()
    for name in keys:
        reports.append(REPORTS[name])
    return reports


def available_reports(context):
    terms = list()
    for report in get_reports():
        name = report.name
        terms.append(SimpleTerm(name, name, report.title))

    return SimpleVocabulary(terms)


class ContentReportsTool(UniqueObject, SimpleItem):

    id = 'content_reports_tool'

    meta_type = 'Content Reports Tool'

    security = ClassSecurityInfo()

    def __init__(self):
        self._report_queue = PersistentList()

    def schedule_report(self, **kwargs):
        self._report_queue.append(PersistentMapping(**kwargs))

    def run_next_report(self):
        queue = self._report_queue
        if len(queue):
            report_request = queue.pop(0)
            try:
                report = get_report(report_request['report'])
                report.run_report(report_request)
            except:
                error = traceback.format_exc()
                self.mail_error(error, report_request)

    def has_reports(self):
        return len(self._report_queue) != 0

    def mail_error(self, error, report_request):
        site = getSite()
        mhost = getToolByName(site, 'MailHost')
        msg = MIMEText(error)
        msg['Subject'] = 'An error occurred while processing your report.'
        mto = list()
        requester_email = report_request.get('requester_email')
        if not requester_email:
            requester_email = report_request['recipients'][0]
        mto.append(requester_email)
        admin_email = api.portal.get_registry_record('plone.email_from_address')
        if admin_email:
            mto.append(admin_email)
            mfrom = admin_email
        else:
            mfrom = requester_email
        mhost.send(msg, mto=mto, mfrom=mfrom)

InitializeClass(ContentReportsTool)
